import logging
import sys
import os
from logging.handlers import RotatingFileHandler


format_str = "[%(asctime)s]  %(levelname)s | %(name)s   -   %(message)s"
date_format = "%Y-%m-%d %H:%M:%S"


if os.getenv("IS_CLOUD", False):
    import google.cloud.logging

    client = google.cloud.logging.Client()
    client.get_default_handler()
    client.setup_logging()
    format_str = "%(levelname)s |  %(name)s   -   %(message)s"


class FixedWidthFormatter(logging.Formatter):
    def format(self, record):
        record.levelname = f"{record.levelname[:7]:<7}"
        record.name = f"{record.name[:16]:<16}"
        return super().format(record)


def create_logger(name: str):
    """Creates a logger using a preset format str. Logs to "All"- and {name}.log if APPDATA_DIR"""
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    log_formatter = FixedWidthFormatter(format_str, datefmt=date_format)
    stream_handler = logging.StreamHandler(sys.stdout)
    stream_handler.setLevel(logging.DEBUG)
    stream_handler.setFormatter(log_formatter)
    logger.addHandler(stream_handler)

    if os.getenv("APPDATA", False) and os.getenv("APPDATA_DIR", False):
        log_folder = f"{os.getenv('APPDATA')}/{os.getenv('APPDATA_DIR')}/logs"
        os.makedirs(log_folder, exist_ok=True)
        log_file_all = f"{log_folder}/All.log"
        log_file = f"{log_folder}/{name}.log"
        file_handler_a = RotatingFileHandler(log_file_all, maxBytes=50 * 1024 * 1024, backupCount=5)
        file_handler_name = RotatingFileHandler(log_file, maxBytes=50 * 1024 * 1024, backupCount=5)
        file_handler_a.setFormatter(log_formatter)
        file_handler_a.setLevel(logging.DEBUG)
        file_handler_name.setFormatter(log_formatter)
        file_handler_name.setLevel(logging.DEBUG)
        logger.addHandler(file_handler_a)
        logger.addHandler(file_handler_name)

    return logger



import os
import sys

from src.common import ROOT_DIR
from src.common_utils.logger import create_logger


log = create_logger("Distribution Helper")


if getattr(sys, "frozen", False):
    log.info(f"Root dir: {ROOT_DIR} - Files in root dir: {os.listdir(ROOT_DIR)}")



import yaml
import dotenv
import os
from os import getenv as secret  # noqa: F401


ROOT_DIR = os.path.abspath(__file__).split("src")[0]

dotenv.load_dotenv()  # load .env file to be imported
stream = open(f"{ROOT_DIR}config.yml", "r", encoding="utf-8")
config_dict = yaml.safe_load(stream)


def config(key):
    # load the config file as unicode strings
    return config_dict[key]



import requests
import json
# from pandas import DataFrame  TODO: uncomment if you use pandas

from src.common_utils.logger import create_logger


class FirebaseClient:
    def __init__(self, realtime_db_url: str | None = None):
        """Firebase client to interact with a realtime database

        Currently only supports unauthenticated access."""
        self.log = create_logger("Firebase")
        self.headers = {"Content-Type": "application/x-www-form-urlencoded"}
        self.database_url = realtime_db_url
        if not self.database_url:
            self.log.error(
                "Could not load firebase project from environment variables. "
                "It will ignore all future interaction"
            )

    def _check_inactive(self) -> bool:
        """Check if we (this Firebase Handler) is active"""
        if not self.database_url:
            self.log.warn("Firebase client is not active. Ignoring request")
        return not self.database_url

    def get_list(self, ref: str, max_results: int = 100, convert_to_list: bool = True):
        """Get a list of child-entries from an object in firebase."""
        if self._check_inactive():
            return
        params = {"orderBy": '"$key"', "limitToFirst": str(max_results)}
        reference_url = f"{self.database_url}/{ref}.json"
        try:
            response = requests.get(url=reference_url, params=params)
            self.log.debug(f"Getting List: {response.text}")
            if convert_to_list:
                list_json = response.json()
                return [list_json[key] for key in list_json]
            else:
                return response.json()
        except Exception as e:
            self.log.error(f"Failed to get list from {ref}: {e}")
            return None

    def update_value(self, ref: str, key: str, value):
        """Update a value of a object entry in firebase"""
        if self._check_inactive():
            return
        data = json.dumps({key: value})
        try:
            response = requests.patch(
                url=f"{self.database_url}/{ref}.json", headers=self.headers, data=data
            )
            self.log.debug(f"Updating Value of {ref}: {response.text}")
        except Exception as e:
            self.log.error(f"Failed to update value of {ref}: {e}")

    def delete_entry(self, ref: str):
        """Delete an object entry from firebase"""
        if self._check_inactive():
            return
        try:
            requests.delete(url=f"{self.database_url}/{ref}.json")
            self.log.debug(f"Deleted entry {ref} from firebase")
        except Exception as e:
            self.log.error(f"Failed to delete entry {ref}: {e}")

    def get_entry(self, ref: str):
        """Get an object entry from firebase"""
        if self._check_inactive():
            return
        try:
            response = requests.get(url=f"{self.database_url}/{ref}.json").json()
            self.log.debug(f"Got entry {ref} from firebase: {response}")
            return response
        except Exception as e:
            self.log.error(f"Failed to get entry {ref}: {e}")
            return None

    def set_entry(self, ref: str, data: dict):
        """Set an object entry in firebase"""
        if self._check_inactive():
            return
        try:
            response = requests.put(
                url=f"{self.database_url}/{ref}.json", headers=self.headers, data=json.dumps(data)
            )
            self.log.debug(f"Set entry {ref} with data {data} in firebase: {response.text}")
        except Exception as e:
            self.log.error(f"Failed to set entry {ref}: {e}")

    # TODO: uncomment if you use pandas
    """ 
    def set_entry_df(self, ref: str, df: DataFrame):
        # Set a dataframe entry in firebase, by first converting it to a dictionary
        if self._check_inactive():
            return
        data = df.to_dict(orient="index")
        try:
            response = requests.put(
                url=f"{self.database_url}/{ref}.json", headers=self.headers, data=json.dumps(data)
            )
            self.log.debug(f"Set df entry {ref} in firebase. Response: {response.text}")
        except Exception as e:
            self.log.error(f"Failed to set df entry {ref}: {e}")

    def get_entry_df(self, ref: str) -> DataFrame | None:
        # Get a dataframe entry from firebase, by converting it from a dictionary
        if self._check_inactive():
            return None
        try:
            data = requests.get(url=f"{self.database_url}/{ref}.json").json()
            self.log.debug(f"Got df entry {ref} from firebase")
            return DataFrame.from_dict(data, orient="index")
        except Exception as e:
            self.log.error(f"Failed to get df entry {ref}: {e}")
            return None
    """


if __name__ == "__main__":
    firebase = FirebaseClient()
    firebase.update_value(ref="To-Do/t-18002271", key="start_datum", value="2022_01_01")
